import { __decorate } from "tslib";
import { LifecycleMixin } from '@genesislcap/foundation-utils';
import { attr, observable } from '@microsoft/fast-element';
import { FoundationElement } from '@microsoft/fast-foundation';
import { ChartDatasource } from './chart-datasource';
import { g2plotChartTemplate as template } from './chart.template';
import { LineChart, AreaChart, BarChart, ColumnChart, DonutChart, PieChart, DualAxesChart, RoseChart, ScatterChart, StockChart, MixChart, } from './models';
import { registerRapidShape } from './settings';
ChartDatasource;
export class G2PlotChart extends LifecycleMixin(FoundationElement) {
    constructor() {
        super(...arguments);
        this.type = 'bar';
        this.legendSelected = {};
        this.initalizeChart = {
            line: (config) => (this.chart = LineChart(this.chartContainer, config)),
            area: (config) => (this.chart = AreaChart(this.chartContainer, config)),
            bar: (config) => (this.chart = BarChart(this.chartContainer, config)),
            rose: (config) => (this.chart = RoseChart(this.chartContainer, config)),
            scatter: (config) => (this.chart = ScatterChart(this.chartContainer, config)),
            pie: (config) => (this.chart = PieChart(this.chartContainer, config)),
            donut: (config) => (this.chart = DonutChart(this.chartContainer, config)),
            column: (config) => (this.chart = ColumnChart(this.chartContainer, config)),
            stock: (config) => (this.chart = StockChart(this.chartContainer, config)),
            dualaxes: (config) => (this.chart = DualAxesChart(this.chartContainer, config)),
            mix: (config) => (this.chart = MixChart(this.chartContainer, config)),
        };
        this.getLegendNameFromData = {
            pie: ({ groupBy }) => groupBy,
            donut: ({ groupBy }) => groupBy,
            bar: ({ series }) => series,
            rose: ({ series }) => series,
        };
        this.initalizeConfiguration = () => {
            var _a, _b;
            const configuration = (_a = this.config) !== null && _a !== void 0 ? _a : {};
            configuration.data = (_b = this.data) !== null && _b !== void 0 ? _b : [[], []];
            configuration.legend = Object.assign(Object.assign({}, configuration.legend), { selected: Object.assign({}, this.legendSelected) });
            return configuration;
        };
    }
    dataChanged(oldValue, newValue) {
        if (!newValue)
            return;
        if (!this.chart && this.isConnected) {
            this.renderChart();
        }
        if (oldValue && oldValue !== newValue) {
            this.chart.changeData(this.data);
        }
    }
    deepClone() {
        const copy = super.deepClone();
        copy.config = structuredClone(this.config);
        copy.data = structuredClone(this.data);
        if (this.legendParser) {
            copy.legendParser = this.legendParser.bind({});
        }
        return copy;
    }
    renderChart() {
        this.initalizeChart[this.type](this.initalizeConfiguration());
        this.chart.render();
    }
    connectedCallback() {
        super.connectedCallback();
        registerRapidShape();
        if (this.data || (this.type == 'mix' && this.config)) {
            this.renderChart();
        }
        this.cacheHiddenLegendItemsCallback = this.cacheHiddenLegendItemsCallback.bind(this);
        this.chart.on('legend-item:click', this.cacheHiddenLegendItemsCallback);
    }
    disconnectedCallback() {
        this.chart.destroy();
    }
    cacheHiddenLegendItemsCallback(...args) {
        if (!(this.type in this.getLegendNameFromData || typeof this.legendParser === 'function')) {
            return;
        }
        const getLegendNameFromData = typeof this.legendParser === 'function'
            ? this.legendParser
            : this.getLegendNameFromData[this.type];
        const displayedSet = new Set(args[0].view.filteredData.map(getLegendNameFromData));
        this.legendSelected = this.data.map(getLegendNameFromData).reduce((acum, legenedName) => (Object.assign(Object.assign({}, acum), { [`${legenedName}`]: displayedSet.has(legenedName) })), {});
        const hasHiddenFields = Object.values(this.legendSelected).includes(false);
        const title = hasHiddenFields ? '+' : '';
        this.$emit('change-title', { title, mode: 'suffix' });
    }
}
__decorate([
    attr
], G2PlotChart.prototype, "type", void 0);
__decorate([
    observable
], G2PlotChart.prototype, "config", void 0);
__decorate([
    observable
], G2PlotChart.prototype, "data", void 0);
export const foundationChartShadowOptions = undefined;
export const defaultChartConfig = {};
export const g2plotChart = G2PlotChart.compose(Object.assign({ baseName: 'g2plot-chart', template, shadowOptions: foundationChartShadowOptions }, defaultChartConfig));
